@@ -22,22 +22,32 @@
  */
 
 #include <vmm_types.h>
-#include <cpu_mmu.h>
+#include <arch_io.h>
+#include <libs/libfdt.h>
+#include <generic_devtree.h>
+#include <generic_mmu.h>
+
+#include <cpu_tlb.h>
+#include <riscv_csr.h>
 
 struct cpu_mmu_entry_ctrl {
-	int num_levels;
+	unsigned long num_levels;
 	u32 pgtbl_count;
-	int *pgtbl_tree;
-	cpu_pte_t *next_pgtbl;
+	arch_pte_t *next_pgtbl;
 	virtual_addr_t pgtbl_base;
 };
 
-extern u8 def_pgtbl[];
-extern int def_pgtbl_tree[];
-#ifdef CONFIG_DEFTERM_EARLY_PRINT
+#ifdef CONFIG_ARCH_GENERIC_DEFTERM_EARLY
 extern u8 defterm_early_base[];
 #endif
 
+#define PGTBL_ROOT_SIZE		(1UL << ARCH_MMU_STAGE1_ROOT_SIZE_ORDER)
+#define PGTBL_ROOT_ENTCNT	(PGTBL_ROOT_SIZE / sizeof(arch_pte_t))
+
+#define PGTBL_INITIAL_COUNT	ARCH_MMU_STAGE1_NONROOT_INITIAL_COUNT
+#define PGTBL_SIZE		(1UL << ARCH_MMU_STAGE1_NONROOT_SIZE_ORDER)
+#define PGTBL_ENTCNT		(PGTBL_SIZE / sizeof(arch_pte_t))
+
 void __attribute__ ((section(".entry")))
     __setup_initial_pgtbl(struct cpu_mmu_entry_ctrl *entry,
 			  virtual_addr_t map_start,
@@ -46,7 +56,7 @@ void __attribute__ ((section(".entry")))
 			  bool writeable)
 {
 	u32 i, index;
-	cpu_pte_t *pgtbl;
+	arch_pte_t *pgtbl;
 	virtual_addr_t page_addr;
 
 	/* align start addresses */
@@ -55,7 +65,7 @@ void __attribute__ ((section(".entry")))
 
 	page_addr = map_start;
 	while (page_addr < map_end) {
-		pgtbl = (cpu_pte_t *)entry->pgtbl_base;
+		pgtbl = (arch_pte_t *)entry->pgtbl_base;
 
 		/* Setup level3 table */
 		if (entry->num_levels < 4) {
@@ -65,28 +75,25 @@ void __attribute__ ((section(".entry")))
 		index = (page_addr & PGTBL_L3_INDEX_MASK) >> PGTBL_L3_INDEX_SHIFT;
 		if (pgtbl[index] & PGTBL_PTE_VALID_MASK) {
 			/* Find level2 table */
-			pgtbl = (cpu_pte_t *)(unsigned long)
+			pgtbl = (arch_pte_t *)(unsigned long)
 				(((pgtbl[index] & PGTBL_PTE_ADDR_MASK)
 				  >> PGTBL_PTE_ADDR_SHIFT)
 				 << PGTBL_PAGE_SIZE_SHIFT);
 		} else {
 			/* Allocate new level2 table */
-			if (entry->pgtbl_count == PGTBL_INITIAL_TABLE_COUNT) {
+			if (entry->pgtbl_count == PGTBL_INITIAL_COUNT) {
 				while (1) ;	/* No initial table available */
 			}
-			for (i = 0; i < PGTBL_TABLE_ENTCNT; i++) {
+			for (i = 0; i < PGTBL_ENTCNT; i++) {
 				entry->next_pgtbl[i] = 0x0ULL;
 			}
-			entry->pgtbl_tree[entry->pgtbl_count] =
-			    ((virtual_addr_t)pgtbl - entry->pgtbl_base) >>
-			    PGTBL_TABLE_SIZE_SHIFT;
 			entry->pgtbl_count++;
 			pgtbl[index] = (virtual_addr_t)entry->next_pgtbl;
 			pgtbl[index] = pgtbl[index] >> PGTBL_PAGE_SIZE_SHIFT;
 			pgtbl[index] = pgtbl[index] << PGTBL_PTE_ADDR_SHIFT;
 			pgtbl[index] |= PGTBL_PTE_VALID_MASK;
 			pgtbl = entry->next_pgtbl;
-			entry->next_pgtbl += PGTBL_TABLE_ENTCNT;
+			entry->next_pgtbl += PGTBL_ENTCNT;
 		}
 #endif
 skip_level3:
@@ -99,28 +106,25 @@ skip_level3:
 		index = (page_addr & PGTBL_L2_INDEX_MASK) >> PGTBL_L2_INDEX_SHIFT;
 		if (pgtbl[index] & PGTBL_PTE_VALID_MASK) {
 			/* Find level1 table */
-			pgtbl = (cpu_pte_t *)(unsigned long)
+			pgtbl = (arch_pte_t *)(unsigned long)
 				(((pgtbl[index] & PGTBL_PTE_ADDR_MASK)
 				  >> PGTBL_PTE_ADDR_SHIFT)
 				 << PGTBL_PAGE_SIZE_SHIFT);
 		} else {
 			/* Allocate new level1 table */
-			if (entry->pgtbl_count == PGTBL_INITIAL_TABLE_COUNT) {
+			if (entry->pgtbl_count == PGTBL_INITIAL_COUNT) {
 				while (1) ;	/* No initial table available */
 			}
-			for (i = 0; i < PGTBL_TABLE_ENTCNT; i++) {
+			for (i = 0; i < PGTBL_ENTCNT; i++) {
 				entry->next_pgtbl[i] = 0x0ULL;
 			}
-			entry->pgtbl_tree[entry->pgtbl_count] =
-			    ((virtual_addr_t)pgtbl - entry->pgtbl_base) >>
-			    PGTBL_TABLE_SIZE_SHIFT;
 			entry->pgtbl_count++;
 			pgtbl[index] = (virtual_addr_t)entry->next_pgtbl;
 			pgtbl[index] = pgtbl[index] >> PGTBL_PAGE_SIZE_SHIFT;
 			pgtbl[index] = pgtbl[index] << PGTBL_PTE_ADDR_SHIFT;
 			pgtbl[index] |= PGTBL_PTE_VALID_MASK;
 			pgtbl = entry->next_pgtbl;
-			entry->next_pgtbl += PGTBL_TABLE_ENTCNT;
+			entry->next_pgtbl += PGTBL_ENTCNT;
 		}
 #endif
 skip_level2:
@@ -132,28 +136,25 @@ skip_level2:
 		index = (page_addr & PGTBL_L1_INDEX_MASK) >> PGTBL_L1_INDEX_SHIFT;
 		if (pgtbl[index] & PGTBL_PTE_VALID_MASK) {
 			/* Find level0 table */
-			pgtbl = (cpu_pte_t *)(unsigned long)
+			pgtbl = (arch_pte_t *)(unsigned long)
 				(((pgtbl[index] & PGTBL_PTE_ADDR_MASK)
 				  >> PGTBL_PTE_ADDR_SHIFT)
 				 << PGTBL_PAGE_SIZE_SHIFT);
 		} else {
 			/* Allocate new level0 table */
-			if (entry->pgtbl_count == PGTBL_INITIAL_TABLE_COUNT) {
+			if (entry->pgtbl_count == PGTBL_INITIAL_COUNT) {
 				while (1) ;	/* No initial table available */
 			}
-			for (i = 0; i < PGTBL_TABLE_ENTCNT; i++) {
+			for (i = 0; i < PGTBL_ENTCNT; i++) {
 				entry->next_pgtbl[i] = 0x0ULL;
 			}
-			entry->pgtbl_tree[entry->pgtbl_count] =
-			    ((virtual_addr_t)pgtbl - entry->pgtbl_base) >>
-			    PGTBL_TABLE_SIZE_SHIFT;
 			entry->pgtbl_count++;
 			pgtbl[index] = (virtual_addr_t)entry->next_pgtbl;
 			pgtbl[index] = pgtbl[index] >> PGTBL_PAGE_SIZE_SHIFT;
 			pgtbl[index] = pgtbl[index] << PGTBL_PTE_ADDR_SHIFT;
 			pgtbl[index] |= PGTBL_PTE_VALID_MASK;
 			pgtbl = entry->next_pgtbl;
-			entry->next_pgtbl += PGTBL_TABLE_ENTCNT;
+			entry->next_pgtbl += PGTBL_ENTCNT;
 		}
 skip_level1:
 
@@ -164,6 +165,8 @@ skip_level1:
 			pgtbl[index] = (page_addr - map_start) + pa_start;
 			pgtbl[index] = pgtbl[index] >> PGTBL_PAGE_SIZE_SHIFT;
 			pgtbl[index] = pgtbl[index] << PGTBL_PTE_ADDR_SHIFT;
+			pgtbl[index] |= PGTBL_PTE_ACCESSED_MASK;
+			pgtbl[index] |= PGTBL_PTE_DIRTY_MASK;
 			pgtbl[index] |= PGTBL_PTE_EXECUTE_MASK;
 			pgtbl[index] |= (writeable) ? PGTBL_PTE_WRITE_MASK : 0;
 			pgtbl[index] |= PGTBL_PTE_READ_MASK;
@@ -206,9 +209,9 @@ skip_level1:
 	extern virtual_addr_t SECTION_END(SECTION)
 
 DECLARE_SECTION(text);
+DECLARE_SECTION(init_text);
 DECLARE_SECTION(cpuinit);
 DECLARE_SECTION(spinlock);
-DECLARE_SECTION(init);
 DECLARE_SECTION(rodata);
 
 #define SETUP_RO_SECTION(ENTRY, SECTION)				\
@@ -218,41 +221,111 @@ DECLARE_SECTION(rodata);
 			     to_load_pa(SECTION_ADDR_START(SECTION)),	\
 			     FALSE)
 
+void __attribute__ ((section(".entry")))
+    __detect_pgtbl_mode(virtual_addr_t load_start, virtual_addr_t load_end,
+			virtual_addr_t exec_start, virtual_addr_t exec_end)
+{
+#ifdef CONFIG_64BIT
+	u32 i, index;
+	unsigned long satp;
+	arch_pte_t *pgtbl =
+		(arch_pte_t *)to_load_pa((virtual_addr_t)&stage1_pgtbl_root);
+
+	/* Clear page table memory */
+	for (i = 0; i < PGTBL_ROOT_ENTCNT; i++) {
+		pgtbl[i] = 0x0ULL;
+	}
+
+	/* Try Sv48 MMU mode */
+	index = (load_start & PGTBL_L3_INDEX_MASK) >> PGTBL_L3_INDEX_SHIFT;
+	pgtbl[index] = load_start & PGTBL_L3_MAP_MASK;
+	pgtbl[index] = pgtbl[index] >> PGTBL_PAGE_SIZE_SHIFT;
+	pgtbl[index] = pgtbl[index] << PGTBL_PTE_ADDR_SHIFT;
+	pgtbl[index] |= PGTBL_PTE_ACCESSED_MASK;
+	pgtbl[index] |= PGTBL_PTE_DIRTY_MASK;
+	pgtbl[index] |= PGTBL_PTE_EXECUTE_MASK;
+	pgtbl[index] |= PGTBL_PTE_WRITE_MASK;
+	pgtbl[index] |= PGTBL_PTE_READ_MASK;
+	pgtbl[index] |= PGTBL_PTE_VALID_MASK;
+	satp = (unsigned long)pgtbl >> PGTBL_PAGE_SIZE_SHIFT;
+	satp |= SATP_MODE_SV48 << SATP_MODE_SHIFT;
+	__sfence_vma_all();
+	csr_write(CSR_SATP, satp);
+	if ((csr_read(CSR_SATP) >> SATP_MODE_SHIFT) == SATP_MODE_SV48) {
+		riscv_stage1_mode = SATP_MODE_SV48;
+	}
+
+	/* Cleanup and disable MMU */
+	for (i = 0; i < PGTBL_ROOT_ENTCNT; i++) {
+		pgtbl[i] = 0x0ULL;
+	}
+	csr_write(CSR_SATP, 0);
+	__sfence_vma_all();
+#endif
+}
+
+virtual_size_t __attribute__ ((section(".entry")))
+    _fdt_size(virtual_addr_t dtb_start)
+{
+	u32 *src = (u32 *)dtb_start;
+
+	if (rev32(src[0]) != FDT_MAGIC) {
+		while (1); /* Hang !!! */
+	}
+
+	return rev32(src[1]);
+}
+
 void __attribute__ ((section(".entry")))
     _setup_initial_pgtbl(virtual_addr_t load_start, virtual_addr_t load_end,
-			 virtual_addr_t exec_start, virtual_addr_t exec_end)
+			 virtual_addr_t exec_start, virtual_addr_t dtb_start)
 {
 	u32 i;
-#ifdef CONFIG_DEFTERM_EARLY_PRINT
+	arch_pte_t *root_pgtbl;
+	virtual_addr_t exec_end = exec_start + (load_end - load_start);
+#ifdef CONFIG_ARCH_GENERIC_DEFTERM_EARLY
 	virtual_addr_t defterm_early_va;
 #endif
-	struct cpu_mmu_entry_ctrl entry = { 0, 0, NULL, NULL, 0 };
+	virtual_addr_t *dt_virt =
+		(virtual_addr_t *)to_load_pa((virtual_addr_t)&devtree_virt);
+	virtual_addr_t *dt_virt_base =
+		(virtual_addr_t *)to_load_pa((virtual_addr_t)&devtree_virt_base);
+	virtual_size_t *dt_virt_size =
+		(virtual_size_t *)to_load_pa((virtual_addr_t)&devtree_virt_size);
+	physical_addr_t *dt_phys_base =
+		(physical_addr_t *)to_load_pa((virtual_addr_t)&devtree_phys_base);
+	struct cpu_mmu_entry_ctrl entry = { 0, 0, NULL, 0 };
 
-	/* For now assume 3-level page table */
-#ifdef CONFIG_64BIT
-	entry.num_levels = 3;
-#else
-	entry.num_levels = 2;
-#endif
+	/* Detect best possible page table mode */
+	__detect_pgtbl_mode(load_start, load_end, exec_start, exec_end);
 
-	/* Init pgtbl_base, pgtbl_tree, and next_pgtbl */
-	entry.pgtbl_tree =
-		(int *)to_load_pa((virtual_addr_t)&def_pgtbl_tree);
-	for (i = 0; i < PGTBL_INITIAL_TABLE_COUNT; i++) {
-		entry.pgtbl_tree[i] = -1;
-	}
-	entry.pgtbl_base = to_load_pa((virtual_addr_t)&def_pgtbl);
-	entry.next_pgtbl = (cpu_pte_t *)entry.pgtbl_base;
-
-	/* Init first pgtbl */
-	for (i = 0; i < PGTBL_TABLE_ENTCNT; i++) {
-		entry.next_pgtbl[i] = 0x0ULL;
+	/* Number of page table levels */
+	switch (riscv_stage1_mode) {
+	case SATP_MODE_SV32:
+		entry.num_levels = 2;
+		break;
+	case SATP_MODE_SV39:
+		entry.num_levels = 3;
+		break;
+	case SATP_MODE_SV48:
+		entry.num_levels = 4;
+		break;
+	default:
+		while (1);
 	}
 
-	entry.pgtbl_count++;
-	entry.next_pgtbl += PGTBL_TABLE_ENTCNT;
+	/* Init pgtbl_base and next_pgtbl */
+	entry.pgtbl_base = to_load_pa((virtual_addr_t)&stage1_pgtbl_root);
+	entry.next_pgtbl =
+		(arch_pte_t *)to_load_pa((virtual_addr_t)&stage1_pgtbl_nonroot);
 
-#ifdef CONFIG_DEFTERM_EARLY_PRINT
+	/* Init root pgtbl */
+	root_pgtbl = (arch_pte_t *)entry.pgtbl_base;
+	for (i = 0; i < PGTBL_ROOT_ENTCNT; i++) {
+		root_pgtbl[i] = 0x0ULL;
+	}
+
+#ifdef CONFIG_ARCH_GENERIC_DEFTERM_EARLY
 	/* Map UART for early defterm
 	 * Note: This is for early debug purpose
 	 */
@@ -260,21 +333,16 @@ void __attribute__ ((section(".entry")))
 	__setup_initial_pgtbl(&entry,
 			      defterm_early_va,
 			      defterm_early_va + PGTBL_L0_BLOCK_SIZE,
-			      (virtual_addr_t)CONFIG_DEFTERM_EARLY_BASE_PA,
+			      CONFIG_ARCH_GENERIC_DEFTERM_EARLY_BASE_PA,
 			      TRUE);
 #endif
 
-	/* Map physical = logical
-	 * Note: This mapping is using at boot time only
-	 */
-	__setup_initial_pgtbl(&entry, load_start, load_end, load_start, TRUE);
-
 	/* Map to logical addresses which are
 	 * covered by read-only linker sections
 	 * Note: This mapping is used at runtime
 	 */
 	SETUP_RO_SECTION(entry, text);
-	SETUP_RO_SECTION(entry, init);
+	SETUP_RO_SECTION(entry, init_text);
 	SETUP_RO_SECTION(entry, cpuinit);
 	SETUP_RO_SECTION(entry, spinlock);
 	SETUP_RO_SECTION(entry, rodata);
@@ -284,4 +352,16 @@ void __attribute__ ((section(".entry")))
 	 * Note: This mapping is used at runtime
 	 */
 	__setup_initial_pgtbl(&entry, exec_start, exec_end, load_start, TRUE);
+
+	/* Compute and save devtree addresses */
+	*dt_phys_base = dtb_start & PGTBL_L0_MAP_MASK;
+	*dt_virt_base = exec_start - _fdt_size(dtb_start);
+	*dt_virt_base &= PGTBL_L0_MAP_MASK;
+	*dt_virt_size = exec_start - *dt_virt_base;
+	*dt_virt = *dt_virt_base + (dtb_start & (PGTBL_L0_BLOCK_SIZE - 1));
+
+	/* Map device tree */
+	__setup_initial_pgtbl(&entry, *dt_virt_base,
+			      *dt_virt_base + *dt_virt_size,
+			      *dt_phys_base, TRUE);
 }
